package arjuna.JavaSim.Distributions;

import java.lang.Math;
import java.io.IOException;
import java.lang.ArithmeticException;

/**
     Devuelve nmeros siguiendo una distribucin Erlang con
     la media y desviacin tpica que se le indique.
*/

public class ErlangStream extends RandomStream {
    /**
       Crea un objeto que permitir generar valores con una distribucin Erlang
       cuya media y desviacin tpica se le indican como parmetros.
    */
    
public ErlangStream (double m, double sd)    {
	super();

	Mean = m;
	StandardDeviation = sd;

	double z = Mean/StandardDeviation;
	k = (long) (z*z);
    }

    /**
       Crea un objeto que permitir generar valores con una distribucin Erlang
       cuya media y desviacin tpica se le indican como parmetros.
       El tercer parmetro es la semilla de la simulacin.
      */
    
public ErlangStream (double m, double sd, int StreamSelect)   {
	super();

	Mean = m;
	StandardDeviation = sd;

	double z = Mean/StandardDeviation;
	k = (long) (z*z);
	for (int ss = 0; ss < StreamSelect*1000; ss++)
	    Uniform();
}

    /**
       Crea un objeto que permitir generar valores con una distribucin Erlang
       cuya media y desviacin tpica se le indican como parmetros. 
       El tercer parmetro es la semilla de la simulacin.  
       Los paramtros cuarto y quinto sern las semillas del generador 
       lineal y congruencial de la clase abstracta 'RandomStream'.
    */
    
public ErlangStream (double m, double sd, int StreamSelect,
		     long MGSeed, long LCGSeed)    {

        super(MGSeed, LCGSeed);

	Mean = m;
	StandardDeviation = sd;

	double z = Mean/StandardDeviation;
	k = (long) (z*z);
	for (int ss = 0; ss < StreamSelect*1000; ss++)
	    Uniform();	
}

    /** 
	Devuelve valores siguiendo la distribucin Erlang.
    */
    
public double getNumber () throws IOException, ArithmeticException    {
	double z = 1.0;
	for (int i = 0; i < k; i++)
	    z *= Uniform();
	
	return -(Mean/k)*Math.log(z);
}
    
private double Mean;
private double StandardDeviation;
private long k;
    
};
