package arjuna.JavaSim.Distributions;

import java.lang.Math;

import java.io.IOException;
import java.lang.ArithmeticException;

/**
   Devuelve nmeros siguiendo una distribucin Hiperexponencial 
   con la media y desviacin tpica que se le indique.
*/

public class HyperExponentialStream extends RandomStream {

    /** 
	Crea un objeto que permitir generar valores con una distribucin 
	Hiperexponencial cuya media y desviacin tpica se le indican 
	como parmetros.
    */
    
public HyperExponentialStream (double m, double sd)    {
	super();

	Mean = m;
	StandardDeviation = sd;

	double cv, z;
	cv = StandardDeviation/Mean;
	z = cv*cv;
	p = 0.5*(1.0-Math.sqrt((z-1.0)/(z+1.0)));
}

    /** 
	Crea un objeto que permitir generar valores con una distribucin 
	Hiperexponencial cuya media y desviacin tpica se le indican 
	como parmetros. El tercer parmetro es la semilla de la simulacin.
      */
    
public HyperExponentialStream (double m, double sd, int StreamSelect)    {
	super();

	Mean = m;
	StandardDeviation = sd;

	double cv, z;
	cv = StandardDeviation/Mean;
	z = cv*cv;
	p = 0.5*(1.0-Math.sqrt((z-1.0)/(z+1.0)));

	for (int ss = 0; ss < StreamSelect*1000; ss++)
	    Uniform();
}
    /**
	Crea un objeto que permitir generar valores con una distribucin 
	Hiperexponencial cuya media y desviacin tpica se le indican 
	como parmetros. El tercer parmetro es la semilla de la simulacin.
	Los parmetros cuarto y quinto sern las semillas del generador lineal 
	y congruencial de la clase abstracta 'RandomStream'.
      */
    
public HyperExponentialStream (double m, double sd, int StreamSelect,
			       long MGSeed, long LCGSeed)    {
	super(MGSeed, LCGSeed);

	Mean = m;
	StandardDeviation = sd;

	double cv, z;
	cv = StandardDeviation/Mean;
	z = cv*cv;
	p = 0.5*(1.0-Math.sqrt((z-1.0)/(z+1.0)));
	
	for (int ss = 0; ss < StreamSelect*1000; ss++)
	    Uniform();	
    }

    /**
       Devuelve valores siguiendo una distribucin Hiperexponencial.       
    */
    
public double getNumber () throws IOException, ArithmeticException    {
	double z = 0;

	if (Uniform() > p)
	    z = Mean/(1.0-p);
	else
	    z = Mean/p;
	
	return -0.5*z*Math.log(Uniform());
}
    
private double Mean;
private double StandardDeviation;
private double p;
    
};
