package arjuna.JavaSim.Distributions;

import java.lang.Math;
import java.io.IOException;
import java.lang.ArithmeticException;

/**
   Devuelve nmeros siguiendo una distribucin Normal
   con la media y su desviacin tpica que se le indique.
*/

public class NormalStream extends RandomStream {

    /**
       Crea un objeto que permitir generar valores con una distribucin
       Normal cuya media y desviacin tpica se le indican como parmetros.
    */
    
public NormalStream (double m, double sd)    {
	super();
	
	Mean  = m;
	StandardDeviation = sd;
	z = 0.0;
}

    /**
       Crea un objeto que permitir generar valores con una distribucin
       Normal cuya media y desviacin tpica se le indican
       como parmetros. El tercer parmetro es la semilla de la simulacin.
      */
    
public NormalStream (double m, double sd, int StreamSelect)
    {
	super();

	Mean  = m;
	StandardDeviation = sd;
	z = 0.0;

	for (int ss = 0; ss < StreamSelect*1000; ss++)
	    Uniform();
    }

    /**
       Crea un objeto que permitir generar valores con una distribucin
       Normal cuya media y desviacin tpica se le indican
       como parmetros. El tercer parmetro es la semilla de la simulacin.
       Los parmetros cuarto y quinto sern las semillas del generador lineal
       y congruencial de la clase abstracta 'RandomStream'.
      */
    
public NormalStream (double m, double sd, int StreamSelect,
		     long MGSeed, long LCGSeed)    {
	super(MGSeed, LCGSeed);

	Mean = m;
	StandardDeviation = sd;
	z = 0.0;

	for (int ss = 0; ss < StreamSelect*1000; ss++)
	    Uniform();
}
    
    /**
       Devuelve valores siguiendo una distribucin Normal. 
       Usa el mtodo polar de Box, Muller y Marsaglia. 
       Tomado de la referencia "Seminumerical Algorithms", Knuth, 
       Addison-Wesley, p.117.
    */
    
public double getNumber () throws IOException, ArithmeticException     {
	// Use the polar method, due to Box, Muller and Marsaglia
	// Taken from Seminumerical Algorithms, Knuth, Addison-Wesley, p.117

	double X2;

	if (z != 0.0)
	{
	    X2 = z;
	    z = 0.0;
	}
	else
	{
	    double S, v1, v2;
	    do
	    {
		v1 = 2.0*Uniform()-1.0;
		v2 = 2.0*Uniform()-1.0;
		S = v1*v1 + v2*v2;
	    } while (S>=1.0);

	    S = Math.sqrt((-2.0*Math.log(S))/S);
	    X2 = v1*S;
	    z  = v2*S;
	}

	return Mean + X2*StandardDeviation;	
}
    
private double Mean;
private double StandardDeviation;
private double z;
  
};
