/*
 * Copyright (C) 1996, 1997, 1998,
 *
 * Department of Computing Science,
 * The University,
 * Newcastle upon Tyne,
 * UK.
 *
 * $Id: Scheduler.java,v 1.3 1998/12/07 08:28:10 nmcl Exp $
 */

package arjuna.JavaSim.Simulation;

import java.util.NoSuchElementException;

public class Scheduler extends Thread {
    
    /**
       Devuelve el tiempo actual de simulacion.
    */
    
    public static double CurrentTime () {
	return Scheduler.SimulatedTime;
    }

    /**
       Reinicia la simulacin. Pone a cero el reloj de tiempo virtual
       de la simulacin y quita todos los procesos de la cola de simulacin.
       Cualquier proceso que est suspendido lanzar la excepcin 
       RestartException, la cual debe ser capturada. Entonces se debera
       poner el proceso en el estado adecuado para reiniciar la simulacin
       antes de llamar el mtodo Cancel del proceso.
    */
    
    public static synchronized void reset () {
	boolean finished = false;
	SimulationProcess tmp = null;
	
	Scheduler._simulationReset = true;
	
	do {
	    do {
		try {
		    tmp = Scheduler.ReadyQueue.Remove();
		}
		catch (NoSuchElementException e){ 
		    finished = true;
		}
		
	    } while (!finished);
	    
	    finished = false;

	    try	{
		tmp = SimulationProcess.allProcesses.Remove();

		/*
		 * Every process must be in Suspend, so we call Resume
		 * and get each one to check whether the simulation is
		 * restarting. If it is, it raises an exception and waits
		 * for the user to cancel the process after setting it
		 * to become ready to restart.
		 */
		
		tmp.Resume();

		/*
		 * Wait for this process to become idle again.
		 */
		
		while (!tmp.idle())
		    Thread.yield();
	    }
	    catch (NoSuchElementException e) {
		finished = true;
	    }
	    
	} while (!finished);

	Scheduler.SimulatedTime = 0.0;
	Scheduler._simulationReset = false;
    }
    
    /**
       Devuelve un booleano indicando si se est reiniciando la simulacin.
    */
    
    public static synchronized boolean simulationReset () {
	return Scheduler._simulationReset;
    }
    
    /**
       Detiene la simulacin.
    */
    
    public static synchronized void stopSimulation () {
	Scheduler.schedulerRunning = false;
    }
    
    /**
       Empieza la simulacin desde el inicio o desde el punto en que se
       detuvo anteriormente.
    */
    
    public static synchronized void startSimulation () {
	Scheduler.schedulerRunning = true;
    }

    protected static synchronized boolean simulationStarted () {
	return Scheduler.schedulerRunning;
    }
    
    private Scheduler ()  {
    }
    
    /**
       Quita el proceso actual activo y pone el primer proceso de la cola 
       de simulacin como proceso activo.

       Es posible que el proceso activo actual pueda quitarse a s mismo de la cola de
       simulacin. Si el proceso que pasa a ser el activo es diferente del anterior
       se devuelve un true. Si el proceso activo que entra es el mismo que estaba se
       devuelve false.
    */
    
    // he aadido la palabra protected para que aparezca la documentacin
    // schedule y unschedule no tenian esta palabra
    protected static synchronized boolean schedule () throws SimulationException {
	if (Scheduler.simulationStarted()) {
	    SimulationProcess p = SimulationProcess.current();
	    
	    try {
		SimulationProcess.Current = Scheduler.ReadyQueue.Remove();
	    }
	    catch (NoSuchElementException e) {
		System.out.println("Simulation queue empty - terminating.");
		System.exit(0);
	    }
	    
	    if (SimulationProcess.Current.evtime() < 0)
		throw new SimulationException("Invalid SimulationProcess wakeup time.");
	    else
		Scheduler.SimulatedTime = SimulationProcess.Current.evtime();
	    
	    if (p != SimulationProcess.Current) {
		SimulationProcess.Current.Resume();
		
		return true;
	    }
	    else
		return false;
	}
	else
	    throw new SimulationException("Simulation not started.");
    }
    
    /** 
	Quita el proceso que recibe como parmetro, de la cola de 
	procesos planificados.
    */   

    protected static synchronized void unschedule (SimulationProcess p) {
	try {
	    Scheduler.ReadyQueue.Remove(p); // remove from queue
	}
	catch (NoSuchElementException e) {}
	
	p.passivate();
    }
    
    protected static double SimulatedTime = 0.0;
    protected static SimulationProcessList ReadyQueue = new SimulationProcessList();
    
    private static boolean schedulerRunning = false;
    private static boolean _simulationReset = false;
    
    static Scheduler theScheduler = new Scheduler();
    
};
