package arjuna.JavaSim.Statistics;

import java.io.*;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.IllegalArgumentException;

/**
   La clase histogram mantiene un nmero fijo de cubos (?buckets).
   Cuando se sobrepasa el nmero de cubos requeridos para mantener
   todos los valores dados, se realiza una operacin de unin. Toma 
   dos cubos y los une segn la poltica seleccionada al crear 
   el histograma.
   Polticas posibles:

   (1) ACCUMULATE -  crea un nuevo bucket con el nombre del bucket
   ms largo, y suma las dos entradas dejndolas en la nueva.

   (2) MEAN - crea un nuevo bucket con la media de los dos nombres y
   suma las dos entradas dejndolas en la nueva.
   
   (3) MAX -  crea un nuevo bucket con el nombre del bucket mayor y 
   el mismo nmero de entradas que ste ltimo. Es decir, duplica el
   bucket mayor.

   (4) MIN - crea un nuevo bucket con el nombre del bucket menor y 
   el mismo nmero de entradas que ste ltimo. Es decir, duplica 
   el bucket menor.

  */

public class Histogram extends PrecisionHistogram {

    /**
       Crea un histograma con un nmero mximo de ndices y especifica 
       qu poltica se seguir al unir cubos (?buckets).
    */
    
public Histogram (long maxIndex, int mergeChoice)    {
	if (maxIndex > 0)
	    maxSize = maxIndex;
	else
	    maxSize = 2;

	merge = mergeChoice;
}

    /**
       Crea un histograma con un mximo de ndices y especifica la poltica 
       de unin "MEAN".
    */
    
public Histogram (long maxIndex)    {

	if (maxIndex > 0)
	    maxSize = maxIndex;
	else
	    maxSize = 2;

	merge = Histogram.MEAN;
}

    /**
       Aade un nuevo valor al histograma. Si el cubo (?bucket) ya existe es 
       incrementado; en otro caso, se crea un nuevo cubo. Esto puede suponer la fusin de 
       buckets existentes para poder hacer sitio.
    */
      
public void setValue (double value) throws IllegalArgumentException    {

	if ((numberOfBuckets() == maxSize) && (!isPresent(value)))
	{
	    try
	    {
		mergeBuckets();
	    }
	    catch (StatisticsException e)
	    {
	    }
	}

	super.setValue(value);	
}

    /**
       Guarda el estado del histograma en el fichero que recibe como parmetro.
    */
    
public boolean saveState (String fileName) throws IOException     {

	FileOutputStream f = new FileOutputStream(fileName);
	DataOutputStream iFile = new DataOutputStream(f);

	boolean res = saveState(iFile);

	f.close();

	return res;
}

    /**
       Guarda el estado del histograma en el 'stream' que recibe como parmetro.
    */
    
public boolean saveState (DataOutputStream oFile) throws IOException     {
	oFile.writeLong(maxSize);
	oFile.writeInt(merge);

	return super.saveState(oFile);
}

    /**
       Restaura el estado del histograma del fichero que recibe como parmetro.
    */
    
public boolean restoreState (String fileName) throws FileNotFoundException, IOException    {
	FileInputStream f = new FileInputStream(fileName);
	DataInputStream oFile = new DataInputStream(f);

	boolean res = restoreState(oFile);

	f.close();

	return res;
}

    /**
       Restaura el estado del histograma del 'stream' que recibe como parmetro. 
    */
    
public boolean restoreState (DataInputStream iFile) throws IOException     {
	maxSize = iFile.readLong();
	merge = iFile.readInt();

	return super.restoreState(iFile);
}

    /**
       Imprime los datos del histograma.
    */
    
public void print ()     {
	System.out.println("Maximum number of buckets "+maxSize);
	System.out.print("Merge choice is ");
	
	switch (merge)
	{
	case Histogram.ACCUMULATE:
	    System.out.println("ACCUMULATE");
	    break;
	case Histogram.MEAN:
	    System.out.println("MEAN");
	    break;
	case Histogram.MAX:
	    System.out.println("MAX");
	    break;
	case Histogram.MIN:
	    System.out.println("MIN");
	    break;
	}
    
	super.print();
}

    /**
       Realiza la fusin de los buckets de acuerdo con 
       el criterio especificado cuando se cre el histograma.
    */
    
protected void mergeBuckets () throws StatisticsException    {
	Bucket newHead = null, ptr = null;
	Bucket index = Head;
	long newLength = 0;
    
	index = super.Head;
    
	while (index != null)
	{
	    Bucket newElement = null;
	
	    // merge buckets in pairs

	    if (index.cdr() != null)
	    {
		newElement = new Bucket(compositeName(index, index.cdr()));
		newElement.size(compositeSize(index, index.cdr()));

		// move on to next pair of buckets

		index = (index.cdr()).cdr();
	    }
	    else
		newElement = new Bucket(index);
	    
	    newLength++;
	    if (newHead != null)
		ptr.setCdr(newElement);
	    else
		newHead = newElement;

	    ptr = newElement;
	}

	index = super.Head;
	ptr = index;

	while (index != null)
	{
	    ptr = index.cdr();
	    index = ptr;
	}
    
	super.Head = newHead;
	super.length = newLength;
}
    
private double compositeName (Bucket a, Bucket b)    {
	switch (merge)
	{
	case ACCUMULATE:
	case MAX:
	    return b.Name();
	case MEAN:
	    return ((a.Name()*a.size() + b.Name()*b.size())/(a.size() + b.size()));
	case MIN:
	    return a.Name();
	default:
	    break;
	}

	return 0.0;	
}
    
private long compositeSize (Bucket a, Bucket b) throws StatisticsException    {
	    switch (merge)
	    {
	    case ACCUMULATE:
		return (a.size() + b.size());
	    case MEAN:
		return (a.size() + b.size());
	    case MAX:
		return b.size();
	    case MIN:
		return a.size();
	    default:
		break;
	    }

	    // shouldn't get here!
	    
	    throw(new StatisticsException("compositeSize switch error."));
}
    
public static final int ACCUMULATE = 0;
public static final int MEAN = 1;
public static final int MAX = 2;
public static final int MIN = 3;

protected long maxSize;
protected int merge;
    
};
