package arjuna.JavaSim.Statistics;

import java.lang.*;
import java.io.*;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.IllegalArgumentException;

/**
   Clase que obtiene la media y otros estadsticos de una muestra.
*/

public class Mean{
    
public Mean ()    {
	reset();
}

    /**
       Aade el parmetro y recalcula del nmero total de muestras, su suma, 
       su media y el valor mnimo y mximo.
    */
    
public void setValue (double value) throws IllegalArgumentException    {
	if (value > _Max)
	    _Max = value;
	if (value < _Min)
	    _Min = value;
	_Sum += value;
	_Number++;
	_Mean = (double) (_Sum/_Number);
}

    /**
       Reinicializa el objeto. Pone la media, el sumatorio y el nmero de 
       muestras a 0.0.
      */
    
public void reset ()    {
	_Max = Float.MIN_VALUE;
	_Min = Float.MAX_VALUE;
	_Sum = _Mean = 0.0;
	_Number = 0;
}

    /**
       Devuelve el nmero de muestras.
    */
    
public int numberOfSamples ()    {
	return _Number;
}

    /**
       Devuelve el mnimo de todas las muestras insertadas. 
    */
    
public double min ()    {
	return _Min;
}

    /**
       Devuelve el mximo de todas las muestras insertadas.
    */
    
public double max ()    {
	return _Max;
}

    /**
       Devuelve el sumatorio de todas las muestras insertadas.
    */
    
public double sum ()    {
	return _Sum;
}

    /**
       Devuelve la media de todas las muestras insertadas.
    */
      
public double mean ()    {
	return _Mean;
}

    /**
       Guarda el estado del estadstico en el fichero que recibe como parmetro.
    */
    
public boolean saveState (String fileName) throws IOException    {
	FileOutputStream f = new FileOutputStream(fileName);
	DataOutputStream oFile = new DataOutputStream(f);

	boolean res = saveState(oFile);

	f.close();

	return res;
}

    /**
       Guarda el estado del estadstico en el 'stream' que recibe 
       como parmetro.
    */
    
public boolean saveState (DataOutputStream oFile) throws IOException    {
	oFile.writeDouble(_Max);
	oFile.writeDouble(_Min);
	oFile.writeDouble(_Sum);
	oFile.writeDouble(_Mean);
	oFile.writeInt(_Number);

	return true;
}

    /**
       Restaura el estado del estadstico a partir fichero que recibe 
       como parmetro.
    */
    
public boolean restoreState (String fileName) throws FileNotFoundException, IOException    {
	FileInputStream f = new FileInputStream(fileName);
	DataInputStream iFile = new DataInputStream(f);

	boolean res = restoreState(iFile);

	f.close();

	return res;
}

    /**
       Restaura el estado del estadstico a partir del 'stream' que recibe 
       como parmetro.
    */
    
public boolean restoreState (DataInputStream iFile) throws IOException    {
	_Max = iFile.readDouble();
	_Min = iFile.readDouble();
	_Sum = iFile.readDouble();
	_Mean = iFile.readDouble();
	_Number = iFile.readInt();
	
	return true;
}

    /**
       Imprime los datos estadsticos recopilados de todas las muestras.
    */
    
public void print ()    {
	System.out.println("Number of samples : "+numberOfSamples());
	System.out.println("Minimum           : "+min());
	System.out.println("Maximum           : "+max());
	System.out.println("Sum               : "+sum());
	System.out.println("Mean              : "+mean());
}
    
protected double _Max;
protected double _Min;
protected double _Sum;
protected double _Mean;
protected int _Number;

};
