package arjuna.JavaSim.Statistics;

import java.lang.*;
import java.io.*;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.IllegalArgumentException;

/**
   Proporciona el modo de obtener los percentiles de una distribucin,
   esto es, los valores menores que lo que devuelve, acumulan una probabilidad 
   que es el porcentaje indicado.
*/

public class Quantile extends PrecisionHistogram {

    /**
       Crea un percentil al 95% de probabilidad.
    */
    
public Quantile ()    {
	qProb = 0.95;
    }

    /**
       Crea un percentil al 95% de probabilidad. Si la probabilidad es menor
       que 0.0 (0% de probabilidad) o mayor que 1.0 (100% de probabilidad), se
       lanza la excepcin 'IllegalArgumentException'.
    */
    
public Quantile (double q) throws IllegalArgumentException    {
	qProb = q;

	if ((q <= 0.0) || (q > 1.0))
	    throw(new IllegalArgumentException("Quantile::Quantile ( "+q+" ) : bad value."));
}

    /**
       Devuelve el percentil.
    */
    
public double getValue ()    {
	double pSamples = numberOfSamples() * qProb;
	long nEntries = 0;
	Bucket ptr = Head, trail = null;

	while ((nEntries < pSamples) && (ptr != null))
	{
	    nEntries += ptr.size();
	    trail = ptr;
	    ptr = ptr.cdr();
	}

	return trail.Name();
}

    /**
       Devuelve el porcentaje del percentil.
    */
    
public double range ()    {
	return qProb;
}

    /**
       Imprime la informacin del percentil.              
    */
    
public void print ()    {
	System.out.println("Quantile precentage : "+qProb);
	System.out.println("Value below which percentage occurs "+this.getValue());
	super.print();
}
    
private double qProb;
   
};
