package arjuna.JavaSim.Statistics;

import java.lang.*;
import java.io.*;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.IllegalArgumentException;

/**
   Crea un histograma simple con un conjunto de cubos (?buckets).
*/

public class SimpleHistogram extends PrecisionHistogram {

    /**
       Crea un histograma con el nmero de cubos (buckets) distribuidos 
       uniformente entre el rango que indican el primer y segundo parmetro.
       El tercer parmetro es el nmero de cubos.
    */
    
public SimpleHistogram (double min, double max, long nbuckets)    {
	if (min < max)
	{
	    minIndex = min;
	    maxIndex = max;
	}
	else
	{
	    minIndex = max;
	    maxIndex = min;
	}

	if (nbuckets > 0)
	    numberBuckets = nbuckets;
	else
	    nbuckets = 1;

	width = (max - min)/numberBuckets;
	super.reset();
}

    /**
       Crea un histograma con el nmero de cubos (buckets) distribuidos 
       uniformente entre el rango que indican el primer y segundo parmetro.
       El tercer parmetro es la anchura.
      */
    
public SimpleHistogram (double min, double max, double w)    {
	if (min < max)
	{
	    minIndex = min;
	    maxIndex = max;
	}
	else
	{
	    minIndex = max;
	    maxIndex = min;
	}

	if (w > 0)
	    width = w;
	else
	    width = 2.0;

	numberBuckets = (long) ((max - min)/width);

	if ((max-min)/width - numberBuckets > 0)
	    numberBuckets++;
	
	super.reset();	
}

    /**
       Aade el parmetro al histograma. Busca el cubo apropiado para 
       incrementarlo.

       @exception IllegalArgumentException Si el parmetro est fuera 
       del rango.
      */    
    
public void setValue (double value) throws IllegalArgumentException    {
	if ((value < minIndex) || (value > maxIndex))
	    throw(new IllegalArgumentException("Value "+value+" is beyond histogram range [ "+minIndex+", "+maxIndex+" ]"));

	for (Bucket ptr = Head; ptr != null; ptr = ptr.cdr())
	{
	    double bucketValue = ptr.Name();

	    if ((value == bucketValue) || (value <= bucketValue + width))
	    {
		super.setValue(ptr.Name());
		return;
	    }
	}

	// shouldn't get here!!

	throw(new IllegalArgumentException("Something went wrong with "+value));
}

    /**
       Vaca el histograma.
    */
    
public void reset ()    {
	double value = minIndex;

	super.reset();
	
	// pre-create buckets with given width

	for (int i = 0; i < numberBuckets; value += width, i++)
	    super.create(value);
}

    /**
       Devuelve el nmero de entradas del cubo (?bucket) que se pasa 
       como primer parmetro.
      */
    
public double sizeByName (double name) throws IllegalArgumentException    {
	if ((name < minIndex) || (name > maxIndex))
	    throw(new IllegalArgumentException("Argument out of range."));

	for (Bucket ptr = Head; ptr != null; ptr = ptr.cdr())
	{
	    double bucketValue = ptr.Name();

	    if ((name == bucketValue) || (name <= bucketValue + width))
		return ptr.size();
	}

	throw(new IllegalArgumentException("Name "+name+" out of range."));
}

    /**
       Devuelve la anchura del cubo (?bucket).
    */
    
public double Width ()    {
	return width;
    }

    /**
       Imprime los datos del histograma.
    */
    
public void print ()    {
	System.out.println("Maximum index range  : "+maxIndex);
	System.out.println("Minimum index range  : "+minIndex);
	System.out.println("Number of buckets    : "+numberBuckets);
	System.out.println("Width of each bucket : "+width);

	super.print();
}

    /**
       Guarda el estado del histograma en el fichero que recibe como parmetro.
    */
    
public boolean saveState (String fileName) throws IOException    {
	FileOutputStream f = new FileOutputStream(fileName);
	DataOutputStream oFile = new DataOutputStream(f);

	boolean res = saveState(oFile);

	f.close();

	return res;	
}
    
    /**
       Guarda el estado del histograma en el 'stream' que recibe como parmetro.
    */
    
public boolean saveState (DataOutputStream oFile) throws IOException    {
	oFile.writeDouble(minIndex);
	oFile.writeDouble(maxIndex);
	oFile.writeDouble(width);
	oFile.writeLong(numberBuckets);
    
	return super.saveState(oFile);
}

    /**
       Restaura el estado del histograma del fichero que recibe como parmetro.       
    */
    
public boolean restoreState (String fileName) throws FileNotFoundException, IOException    {
	FileInputStream f = new FileInputStream(fileName);
	DataInputStream iFile = new DataInputStream(f);

	boolean res = restoreState(iFile);

	f.close();

	return res;	
}

    /**
       Restaura el estado del histograma del 'stream' que recibe como parmetro.
    */
    
public boolean restoreState (DataInputStream iFile) throws IOException    {
	minIndex = iFile.readDouble();
	maxIndex = iFile.readDouble();
	width = iFile.readDouble();
	numberBuckets = iFile.readLong();

	return super.restoreState(iFile);
}

private double minIndex;
private double maxIndex;
private double width;
private long numberBuckets;

};
