package arjuna.JavaSim.Statistics;

import java.io.*;
import arjuna.JavaSim.Simulation.*;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.IllegalArgumentException;


/**
   Clase que obtiene el valor medio durante un intervalo de tiempo de 
   simulacin.
*/

public class TimeVariance extends Variance {
    
public TimeVariance ()    {
	reset();
}

    /**
       Reinicializa el objeto. Pone todos sus estadsticos a 0.0 y el 
       extremo inferior del intervalo de tiempo a 0.0.
    */
    
public void reset ()    {
	first = true;
	startTime = currentValue = 0.0;
	stime = total = 0.0;
	super.reset();
}

    /**
       Aade el parmetro y recalcula todos los estadsticos.
    */
    
public void setValue (double value) throws IllegalArgumentException    {
	super.setValue(value);

	if (!first)
	{
	    total += area();
	    if (value == currentValue)
		return;
	}
	else
	{
	    first = false;
	    startTime = SimulationProcess.CurrentTime();
	}

	store(value);
}

    /**
       Devuelve el valor medio, tomando como final del intervalo, el tiempo 
       de simulacin actual.
    */
    
public double timeAverage ()    {
	if (first || (SimulationProcess.CurrentTime() - startTime) == 0)
	    return 0.0;

	return ((total + area())/(SimulationProcess.CurrentTime() - startTime));
}
    
    /**
       Guarda el estado del estadstico en el fichero que recibe como parmetro.
    */
    
public boolean saveState (String fileName) throws IOException     {
	FileOutputStream f = new FileOutputStream(fileName);
	DataOutputStream oFile = new DataOutputStream(f);

	boolean res = saveState(oFile);

	f.close();

	return res;
}

    /**
       Guarda el estado del estadstico  en el 'stream' que recibe como parmetro.
    */
    
public boolean saveState (DataOutputStream oFile) throws IOException
    {
	oFile.writeBoolean(first);
	oFile.writeDouble(startTime);
	oFile.writeDouble(currentValue);
	oFile.writeDouble(stime);
	oFile.writeDouble(total);
    
	return super.saveState(oFile);
    }

    /**
       Restaura el estado del estadstico del fichero que recibe como parmetro.
    */
    
public boolean restoreState (String fileName) throws FileNotFoundException, IOException    {
	FileInputStream f = new FileInputStream(fileName);
	DataInputStream iFile = new DataInputStream(f);

	boolean res = restoreState(iFile);

	f.close();

	return res;
}
    
    /**
       Restaura el estado del estadstico del 'stream' que recibe como parmetro. 
    */
    
public boolean restoreState (DataInputStream iFile) throws IOException    {
	first = iFile.readBoolean();
	startTime = iFile.readDouble();
	currentValue = iFile.readDouble();
	stime = iFile.readDouble();
	total = iFile.readDouble();

	return true;
}
    
private double area ()    {
	return (currentValue * (SimulationProcess.CurrentTime() - stime));
}
    
private void store (double value)    {
	currentValue = value;
	stime = SimulationProcess.CurrentTime();
}

private boolean first;
private double startTime;
private double currentValue;
private double stime;
private double total;

};
