package arjuna.JavaSim.Statistics;

import java.io.*;
import java.lang.Math;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.IllegalArgumentException;

/**
   Extiende a la clase Mean, obteniendo la varianza de las muestras.
*/

public class Variance extends Mean{
    
public Variance ()    {
	reset();
}

    /**
       Aade el parmetro y recalcula la varianza.      
    */
    
public void setValue (double value) throws IllegalArgumentException    {
	_sqr += value*value;
	super.setValue(value);
    }

    /**
       Reinicia el objeto. Pone la varianza y el resto de estadsticos a 0.0.
    */
    
public void reset ()    {
	_sqr = 0.0;
	super.reset();
}

    /**
       Devuelve la varianza.
    */
    
public double variance ()    {
	if (_Number > 1)
	    return ((_sqr - ((_Sum * _Sum) / _Number)) / (_Number -1));
	else
	    return 0.0;
}

    /**
       Devuelve la desviacin tpica de las muestras.
    */
      
public double stdDev ()    {
	if (_Number == 0 || variance() <= 0)
	    return 0.0;
	else
	    return Math.sqrt(variance());	
}

    /**
       No est implementado.
    */
      
public double confidence (double value)    {
	System.out.println("Variance::confidence not implemented yet.");
	return 0.0;	
}

    /**
       Imprime los datos estadsticos recopilados de todas las muestras.
    */
    
public void print ()    {
	System.out.println("Variance          : "+variance());
	System.out.println("Standard Deviation: "+stdDev());

	super.print();
}

    /**
       Guarda el estado del estadstico en el fichero que recibe como parmetro.
    */
    
public boolean saveState (String fileName) throws IOException    {
	FileOutputStream f = new FileOutputStream(fileName);
	DataOutputStream oFile = new DataOutputStream(f);

	boolean res = saveState(oFile);

	f.close();

	return res;
}

    /**
       Guarda el estado del estadstico en el 'stream' que recibe como parmetro.
    */
    
public boolean saveState (DataOutputStream oFile) throws IOException    {
	oFile.writeDouble(_sqr);
	return super.saveState(oFile);	
}

    /**
       Restaura el estado del estadstico del fichero que recibe como parmetro.
      */
    
public boolean restoreState (String fileName) throws FileNotFoundException, IOException    {
	FileInputStream f = new FileInputStream(fileName);
	DataInputStream iFile = new DataInputStream(f);

	boolean res = restoreState(iFile);

	f.close();

	return res;
}

    /**
       Restaura el estado del estadstico del 'stream' que recibe como parmetro.
    */
    
public boolean restoreState (DataInputStream iFile) throws IOException    {
	_sqr = iFile.readDouble();

	return super.restoreState(iFile);
}
    
private double _sqr;

};
